@@ -32,8 +32,6 @@
 #include <string.h>
 #endif
 
-#include <libxfce4ui/libxfce4ui.h>
-
 #include "thunar/thunar-dialogs.h"
 #include "thunar/thunar-gtk-extensions.h"
 #include "thunar/thunar-icon-factory.h"
@@ -44,10 +42,14 @@
 #include "thunar/thunar-private.h"
 #include "thunar/thunar-util.h"
 
+#include <libxfce4ui/libxfce4ui.h>
 
 
-static void          thunar_dialogs_select_filename      (GtkWidget *entry);
-static void          thunar_dialogs_shrink_height        (GtkWidget *dialog);
+
+static void
+thunar_dialogs_select_filename (GtkWidget *entry);
+static void
+thunar_dialogs_shrink_height (GtkWidget *dialog);
 
 
 
@@ -81,7 +83,7 @@ thunar_dialogs_shrink_height (GtkWidget *dialog)
  * Return value: the filename entered by the user or %NULL if the user
  *               cancelled the dialog.
  **/
-gchar*
+gchar *
 thunar_dialogs_show_create (gpointer     parent,
                             const gchar *content_type,
                             const gchar *filename,
@@ -110,7 +112,7 @@ thunar_dialogs_show_create (gpointer     parent,
                                         GTK_DIALOG_MODAL
                                         | GTK_DIALOG_DESTROY_WITH_PARENT,
                                         _("_Cancel"), GTK_RESPONSE_CANCEL,
-                                        _("C_reate"), GTK_RESPONSE_OK,
+                                          _("C_reate"), GTK_RESPONSE_OK,
                                         NULL);
 
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
@@ -161,7 +163,7 @@ thunar_dialogs_show_create (gpointer     parent,
   gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (filename_input), 1, row, 1, 1);
   thunar_gtk_label_set_a11y_relation (GTK_LABEL (label),
                                       GTK_WIDGET (xfce_filename_input_get_entry (filename_input)));
-  gtk_widget_show_all ( GTK_WIDGET (filename_input));
+  gtk_widget_show_all (GTK_WIDGET (filename_input));
 
   /* ensure that the sensitivity of the Create button is set correctly */
   xfce_filename_input_check (filename_input);
@@ -177,7 +179,7 @@ thunar_dialogs_show_create (gpointer     parent,
 
   /* shrink the dialog again, after some filename error was fixed */
   g_signal_connect_swapped (filename_input, "text-valid", G_CALLBACK (thunar_dialogs_shrink_height), dialog);
- 
+
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
     {
       /* determine the chosen filename */
@@ -230,17 +232,16 @@ thunar_dialogs_show_rename_file (gpointer               parent,
   GtkWidget         *label;
   GtkWidget         *image;
   GtkWidget         *grid;
+  XfceFilenameInput *filename_input;
+  GtkEntry          *filename_input_entry;
   GtkWindow         *window;
+  GdkScreen         *screen;
   GdkPixbuf         *icon;
   cairo_surface_t   *surface;
-  GdkScreen         *screen;
   gchar             *title;
   gint               response;
-  PangoLayout       *layout;
-  gint               layout_width;
-  gint               layout_offset;
+  gint               dialog_width;
   gint               parent_width = 500;
-  XfceFilenameInput *filename_input;
   gint               row = 0;
 
   _thunar_return_val_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WINDOW (parent), FALSE);
@@ -259,7 +260,7 @@ thunar_dialogs_show_rename_file (gpointer               parent,
                                         GTK_DIALOG_MODAL
                                         | GTK_DIALOG_DESTROY_WITH_PARENT,
                                         _("_Cancel"), GTK_RESPONSE_CANCEL,
-                                        _("_Rename"), GTK_RESPONSE_OK,
+                                          _("_Rename"), GTK_RESPONSE_OK,
                                         NULL);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
   g_free (title);
@@ -305,6 +306,7 @@ thunar_dialogs_show_rename_file (gpointer               parent,
 
   /* set up the widget for entering the filename */
   filename_input = g_object_new (XFCE_TYPE_FILENAME_INPUT, "original-filename", filename, NULL);
+  filename_input_entry = xfce_filename_input_get_entry (filename_input);
   gtk_widget_set_hexpand (GTK_WIDGET (filename_input), TRUE);
   gtk_widget_set_valign (GTK_WIDGET (filename_input), GTK_ALIGN_CENTER);
 
@@ -314,40 +316,39 @@ thunar_dialogs_show_rename_file (gpointer               parent,
                             gtk_dialog_get_widget_for_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK));
   g_signal_connect_swapped (filename_input, "text-valid", G_CALLBACK (xfce_filename_input_sensitise_widget),
                             gtk_dialog_get_widget_for_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK));
-                          
-                            /* next row */
+
+  /* next row */
   row++;
 
   gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (filename_input), 1, row, 1, 1);
-  thunar_gtk_label_set_a11y_relation (GTK_LABEL (label),
-                                      GTK_WIDGET (xfce_filename_input_get_entry (filename_input)));
-  gtk_widget_show_all ( GTK_WIDGET (filename_input));
+  thunar_gtk_label_set_a11y_relation (GTK_LABEL (label), GTK_WIDGET (filename_input_entry));
+  gtk_widget_show_all (GTK_WIDGET (filename_input));
 
   /* ensure that the sensitivity of the Create button is set correctly */
   xfce_filename_input_check (filename_input);
 
   /* If it is no directory, select the filename without the extension */
   if (thunar_file_is_directory (file))
-    gtk_editable_select_region (GTK_EDITABLE (xfce_filename_input_get_entry (filename_input)), 0, -1);
+    gtk_editable_select_region (GTK_EDITABLE (filename_input_entry), 0, -1);
   else
-    thunar_dialogs_select_filename (GTK_WIDGET (xfce_filename_input_get_entry (filename_input)));
+    thunar_dialogs_select_filename (GTK_WIDGET (filename_input_entry));
 
-  /* get the size the entry requires to render the full text */
-  layout = gtk_entry_get_layout (xfce_filename_input_get_entry (filename_input));
-  pango_layout_get_pixel_size (layout, &layout_width, NULL);
-  gtk_entry_get_layout_offsets (xfce_filename_input_get_entry (filename_input), &layout_offset, NULL);
-  layout_width += (layout_offset * 2) + (12 * 4) + 48; /* 12px free space in entry */
+  /* change the width of the input entry to be about the right size for N chars (filename length) */
+  gtk_entry_set_max_width_chars (filename_input_entry, g_utf8_strlen (filename, -1));
+
+  /* dialog window width */
+  gtk_window_get_size (GTK_WINDOW (dialog), &dialog_width, NULL);
 
   /* parent window width */
   if (G_LIKELY (window != NULL))
     {
       /* keep below 90% of the parent window width */
-      gtk_window_get_size (GTK_WINDOW (window), &parent_width, NULL);
+      gtk_window_get_size (window, &parent_width, NULL);
       parent_width *= 0.90f;
     }
 
   /* resize the dialog to make long names fit as much as possible */
-  gtk_window_set_default_size (GTK_WINDOW (dialog), CLAMP (layout_width, 300, parent_width), -1);
+  gtk_window_set_default_size (GTK_WINDOW (dialog), CLAMP (dialog_width, 300, parent_width), -1);
 
   /* automatically close the dialog when the file is destroyed */
   g_signal_connect_swapped (G_OBJECT (file), "destroy",
@@ -355,7 +356,7 @@ thunar_dialogs_show_rename_file (gpointer               parent,
 
   /* shrink the dialog again, after some filename error was fixed */
   g_signal_connect_swapped (filename_input, "text-valid", G_CALLBACK (thunar_dialogs_shrink_height), dialog);
- 
+
   /* run the dialog */
   response = gtk_dialog_run (GTK_DIALOG (dialog));
   if (G_LIKELY (response == GTK_RESPONSE_OK))
@@ -404,30 +405,27 @@ thunar_dialogs_show_about (GtkWindow   *parent,
                            const gchar *format,
                            ...)
 {
-  static const gchar *artists[] =
-  {
+  static const gchar *artists[] = {
     "Young Hahn <youngjin.hahn@gmail.com>",
     NULL,
   };
 
-  static const gchar *authors[] =
-  {
+  static const gchar *authors[] = {
     "Benedikt Meurer <benny@xfce.org>",
     "Jannis Pohlmann <jannis@xfce.org>",
     "Nick Schermer <nick@xfce.org>",
     NULL,
   };
 
-  static const gchar *documenters[] =
-  {
+  static const gchar *documenters[] = {
     "Benedikt Meurer <benny@xfce.org>",
     "Jannis Pohlmann <jannis@xfce.org>",
     "Nick Schermer <nick@xfce.org>",
     NULL,
   };
 
-  va_list    args;
-  gchar     *comments;
+  va_list args;
+  gchar  *comments;
 
   _thunar_return_if_fail (parent == NULL || GTK_IS_WINDOW (parent));
 
@@ -524,7 +522,7 @@ thunar_dialogs_show_error (gpointer      parent,
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s.", error->message);
 
   children = gtk_container_get_children (
-    GTK_CONTAINER (gtk_message_dialog_get_message_area (GTK_MESSAGE_DIALOG (dialog))));
+  GTK_CONTAINER (gtk_message_dialog_get_message_area (GTK_MESSAGE_DIALOG (dialog))));
 
   /* enable wrap for labels */
   for (lp = children; lp != NULL; lp = lp->next)
@@ -608,8 +606,7 @@ thunar_dialogs_show_job_ask (GtkWindow        *parent,
 
   /* allocate the question message dialog */
   message = gtk_message_dialog_new (parent,
-                                    GTK_DIALOG_MODAL |
-                                    GTK_DIALOG_DESTROY_WITH_PARENT,
+                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                     GTK_MESSAGE_QUESTION,
                                     GTK_BUTTONS_NONE,
                                     "%s", primary->str);
@@ -719,23 +716,10 @@ thunar_dialogs_show_job_ask (GtkWindow        *parent,
 
 
 
-static void thunar_dialogs_show_job_ask_replace_callback (GtkWidget *button,
-                                                          gpointer   user_data)
-{
-  gint response;
-
-  _thunar_return_if_fail (GTK_IS_DIALOG (user_data));
-
-  response = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (button), "response-id"));
-  gtk_dialog_response (GTK_DIALOG (user_data), response);
-}
-
-
-
 static void
-thunar_dialog_image_redraw (GtkWidget           *image,
-                            ThunarThumbnailSize  size,
-                            ThunarFile          *file)
+thunar_dialog_image_redraw (GtkWidget          *image,
+                            ThunarThumbnailSize size,
+                            ThunarFile         *file)
 {
   ThunarIconFactory *icon_factory;
   GtkIconTheme      *icon_theme;
@@ -775,27 +759,24 @@ thunar_dialog_image_redraw (GtkWidget           *image,
 ThunarJobResponse
 thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
                                      ThunarFile *src_file,
-                                     ThunarFile *dst_file)
+                                     ThunarFile *dst_file,
+                                     gboolean    multiple_files)
 {
   ThunarIconFactory *icon_factory;
   ThunarPreferences *preferences;
   ThunarDateStyle    date_style;
+  ThunarFile        *parent_file;
   GtkIconTheme      *icon_theme;
   GtkWidget         *dialog;
   GtkWidget         *grid;
   GtkWidget         *image, *src_image, *dst_image;
   GtkWidget         *label;
   GtkWidget         *content_area;
-  GtkWidget         *cancel_button;
-  GtkWidget         *button_box;
-  GtkWidget         *skipall_button;
-  GtkWidget         *skip_button;
-  GtkWidget         *replaceall_button;
-  GtkWidget         *replace_button;
-  GtkWidget         *renameall_button;
-  GtkWidget         *rename_button;
+  GtkWidget         *action_area;
+  GtkWidget         *check_button;
   GdkPixbuf         *icon;
   cairo_surface_t   *surface;
+  const gchar       *parent_string = "";
   gchar             *date_custom_style;
   gchar             *date_string;
   gchar             *size_string;
@@ -804,6 +785,7 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   gboolean           file_size_binary;
   gint               row = 0;
   gint               scale_factor;
+  gint               width;
 
   _thunar_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), THUNAR_JOB_RESPONSE_CANCEL);
   _thunar_return_val_if_fail (THUNAR_IS_FILE (src_file), THUNAR_JOB_RESPONSE_CANCEL);
@@ -817,14 +799,24 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   g_object_unref (G_OBJECT (preferences));
 
   /* setup the confirmation dialog */
-  dialog = gtk_dialog_new();
-  gtk_window_set_title (GTK_WINDOW (dialog), _("Confirm to replace files"));
-  gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
-  gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
-  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+  dialog = gtk_dialog_new_with_buttons (_("Confirm to replace files"),
+                                        parent,
+                                        GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                          _("_Cancel"), THUNAR_JOB_RESPONSE_CANCEL,
+                                            _("_Skip"), THUNAR_JOB_RESPONSE_SKIP,
+                                              _("Re_name"), THUNAR_JOB_RESPONSE_RENAME,
+                                                _("_Replace"), THUNAR_JOB_RESPONSE_REPLACE,
+                                        NULL);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), THUNAR_JOB_RESPONSE_REPLACE);
+
   content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+  action_area = gtk_dialog_get_action_area (GTK_DIALOG (dialog));
+  G_GNUC_END_IGNORE_DEPRECATIONS
+  gtk_button_box_set_layout (GTK_BUTTON_BOX (action_area), GTK_BUTTONBOX_SPREAD);
+  gtk_widget_set_margin_bottom (GTK_WIDGET (action_area), 3);
+
   if (parent != NULL)
     scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (parent));
   else
@@ -841,52 +833,13 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   gtk_box_pack_start (GTK_BOX (content_area), grid, TRUE, FALSE, 0);
   gtk_widget_show (grid);
 
-  /* set up the action area buttons ourself */
-  button_box = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
-
-  cancel_button     = gtk_button_new_with_mnemonic (_("_Cancel"));
-  skipall_button    = gtk_button_new_with_mnemonic (_("S_kip All"));
-  skip_button       = gtk_button_new_with_mnemonic (_("_Skip"));
-  replaceall_button = gtk_button_new_with_mnemonic (_("Replace _All"));
-  replace_button    = gtk_button_new_with_mnemonic (_("_Replace"));
-  renameall_button  = gtk_button_new_with_mnemonic (_("Rena_me All"));
-  rename_button     = gtk_button_new_with_mnemonic (_("Re_name"));
-
-  g_signal_connect (cancel_button,      "clicked", G_CALLBACK (thunar_dialogs_show_job_ask_replace_callback), dialog);
-  g_signal_connect (skipall_button,     "clicked", G_CALLBACK (thunar_dialogs_show_job_ask_replace_callback), dialog);
-  g_signal_connect (skip_button,        "clicked", G_CALLBACK (thunar_dialogs_show_job_ask_replace_callback), dialog);
-  g_signal_connect (replaceall_button,  "clicked", G_CALLBACK (thunar_dialogs_show_job_ask_replace_callback), dialog);
-  g_signal_connect (replace_button,     "clicked", G_CALLBACK (thunar_dialogs_show_job_ask_replace_callback), dialog);
-  g_signal_connect (renameall_button,   "clicked", G_CALLBACK (thunar_dialogs_show_job_ask_replace_callback), dialog);
-  g_signal_connect (rename_button,      "clicked", G_CALLBACK (thunar_dialogs_show_job_ask_replace_callback), dialog);
-
-  g_object_set_data (G_OBJECT (cancel_button),     "response-id", GINT_TO_POINTER (GTK_RESPONSE_CANCEL));
-  g_object_set_data (G_OBJECT (skipall_button),    "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_SKIP_ALL));
-  g_object_set_data (G_OBJECT (skip_button),       "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_SKIP));
-  g_object_set_data (G_OBJECT (replaceall_button), "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_REPLACE_ALL));
-  g_object_set_data (G_OBJECT (replace_button),    "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_REPLACE));
-  g_object_set_data (G_OBJECT (renameall_button),  "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_RENAME_ALL));
-  g_object_set_data (G_OBJECT (rename_button),     "response-id", GINT_TO_POINTER (THUNAR_JOB_RESPONSE_RENAME));
-
-  gtk_container_add (GTK_CONTAINER (button_box), cancel_button);
-  gtk_container_add (GTK_CONTAINER (button_box), skipall_button);
-  gtk_container_add (GTK_CONTAINER (button_box), skip_button);
-  gtk_container_add (GTK_CONTAINER (button_box), replaceall_button);
-  gtk_container_add (GTK_CONTAINER (button_box), replace_button);
-  gtk_container_add (GTK_CONTAINER (button_box), renameall_button);
-  gtk_container_add (GTK_CONTAINER (button_box), rename_button);
-  gtk_container_add (GTK_CONTAINER (content_area), button_box);
-  gtk_widget_set_halign (button_box, GTK_ALIGN_CENTER);
-  gtk_box_set_spacing (GTK_BOX (button_box), 5);
-  gtk_widget_show_all (button_box);
-
   image = gtk_image_new_from_icon_name ("stock_folder-copy", GTK_ICON_SIZE_BUTTON);
   gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
   gtk_widget_set_valign (image, GTK_ALIGN_START);
-  gtk_widget_set_margin_start (GTK_WIDGET(image), 6);
-  gtk_widget_set_margin_end (GTK_WIDGET(image), 6);
-  gtk_widget_set_margin_top (GTK_WIDGET(image), 6);
-  gtk_widget_set_margin_bottom (GTK_WIDGET(image), 6);
+  gtk_widget_set_margin_start (GTK_WIDGET (image), 6);
+  gtk_widget_set_margin_end (GTK_WIDGET (image), 6);
+  gtk_widget_set_margin_top (GTK_WIDGET (image), 6);
+  gtk_widget_set_margin_bottom (GTK_WIDGET (image), 6);
   gtk_widget_set_vexpand (image, TRUE);
   gtk_grid_attach (GTK_GRID (grid), image, 0, row, 1, 1);
   gtk_widget_show (image);
@@ -914,28 +867,39 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
 
   label = gtk_label_new (text);
   gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
-  gtk_label_set_attributes (GTK_LABEL (label), thunar_pango_attr_list_big ());
+  gtk_label_set_attributes (GTK_LABEL (label), thunar_pango_attr_list_big_bold ());
+  gtk_label_set_selectable (GTK_LABEL (label), TRUE);
+  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+  gtk_label_set_line_wrap_mode (GTK_LABEL (label), PANGO_WRAP_CHAR);
+  thunar_gtk_label_disable_hyphens (GTK_LABEL (label));
   gtk_widget_set_hexpand (label, TRUE);
-  gtk_label_set_selectable(GTK_LABEL (label), TRUE);
   gtk_grid_attach (GTK_GRID (grid), label, 1, row, 2, 1);
   gtk_widget_show (label);
   g_free (text);
 
+  parent_file = thunar_file_get_parent (dst_file, NULL);
+  if (parent_file != NULL)
+    parent_string = thunar_file_get_basename (parent_file);
+
   if (thunar_file_is_symlink (dst_file))
     /* TRANSLATORS: First part of replace dialog sentence */
-    text = g_strdup_printf (_("Do you want to replace the link"));
+    text = g_strdup_printf (_("Replace the link in \"%s\""), parent_string);
   else if (thunar_file_is_directory (dst_file))
     /* TRANSLATORS: First part of replace dialog sentence */
-    text = g_strdup_printf (_("Do you want to replace the existing folder"));
+    text = g_strdup_printf (_("Replace the existing folder in \"%s\""), parent_string);
   else
     /* TRANSLATORS: First part of replace dialog sentence */
-    text = g_strdup_printf (_("Do you want to replace the existing file"));
+    text = g_strdup_printf (_("Replace the existing file in \"%s\""), parent_string);
+
+  if (parent_file != NULL)
+    g_object_unref (parent_file);
 
   /* next row */
   row++;
 
   label = gtk_label_new (text);
   gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
+  gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_MIDDLE);
   gtk_widget_set_hexpand (label, TRUE);
   gtk_grid_attach (GTK_GRID (grid), label, 1, row, 2, 1);
   gtk_widget_show (label);
@@ -947,10 +911,10 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   icon = thunar_icon_factory_load_file_icon (icon_factory, dst_file, THUNAR_FILE_ICON_STATE_DEFAULT, 48, scale_factor, FALSE, NULL);
   surface = gdk_cairo_surface_create_from_pixbuf (icon, scale_factor, NULL);
   dst_image = gtk_image_new_from_surface (surface);
-  gtk_widget_set_margin_start (GTK_WIDGET(dst_image), 6);
-  gtk_widget_set_margin_end (GTK_WIDGET(dst_image), 6);
-  gtk_widget_set_margin_top (GTK_WIDGET(dst_image), 6);
-  gtk_widget_set_margin_bottom (GTK_WIDGET(dst_image), 6);
+  gtk_widget_set_margin_start (GTK_WIDGET (dst_image), 6);
+  gtk_widget_set_margin_end (GTK_WIDGET (dst_image), 6);
+  gtk_widget_set_margin_top (GTK_WIDGET (dst_image), 6);
+  gtk_widget_set_margin_bottom (GTK_WIDGET (dst_image), 6);
   gtk_grid_attach (GTK_GRID (grid), dst_image, 1, row, 1, 1);
   g_object_unref (G_OBJECT (icon));
   cairo_surface_destroy (surface);
@@ -997,10 +961,10 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   icon = thunar_icon_factory_load_file_icon (icon_factory, src_file, THUNAR_FILE_ICON_STATE_DEFAULT, 48, scale_factor, FALSE, NULL);
   surface = gdk_cairo_surface_create_from_pixbuf (icon, scale_factor, NULL);
   src_image = gtk_image_new_from_surface (surface);
-  gtk_widget_set_margin_start (GTK_WIDGET(src_image), 6);
-  gtk_widget_set_margin_end (GTK_WIDGET(src_image), 6);
-  gtk_widget_set_margin_top (GTK_WIDGET(src_image), 6);
-  gtk_widget_set_margin_bottom (GTK_WIDGET(src_image), 6);
+  gtk_widget_set_margin_start (GTK_WIDGET (src_image), 6);
+  gtk_widget_set_margin_end (GTK_WIDGET (src_image), 6);
+  gtk_widget_set_margin_top (GTK_WIDGET (src_image), 6);
+  gtk_widget_set_margin_bottom (GTK_WIDGET (src_image), 6);
   gtk_grid_attach (GTK_GRID (grid), src_image, 1, row, 1, 1);
   g_object_unref (G_OBJECT (icon));
   cairo_surface_destroy (surface);
@@ -1020,9 +984,44 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   g_free (size_string);
   g_free (date_string);
   g_free (text);
+  g_free (date_custom_style);
+
+  if (multiple_files)
+    {
+      /* next row */
+      row++;
+
+      check_button = gtk_check_button_new_with_mnemonic (_("_Apply the action to all files and folders"));
+      gtk_widget_set_margin_top (check_button, 6);
+      gtk_grid_attach (GTK_GRID (grid), check_button, 0, row, 3, 1);
+      gtk_widget_show (check_button);
+    }
+
+  /* prevent long filenames from pushing the dialog width over a certain value */
+  gtk_window_get_size (GTK_WINDOW (dialog), &width, NULL);
+  if (width > 600)
+    gtk_window_resize (GTK_WINDOW (dialog), 600, 1);
 
   /* run the dialog */
   response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+  /* translate GTK responses */
+  if (G_UNLIKELY (response < 0))
+    response = THUNAR_JOB_RESPONSE_CANCEL;
+
+  if (multiple_files)
+    {
+      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check_button)))
+        {
+          if (response == THUNAR_JOB_RESPONSE_SKIP)
+            response = THUNAR_JOB_RESPONSE_SKIP_ALL;
+          else if (response == THUNAR_JOB_RESPONSE_REPLACE)
+            response = THUNAR_JOB_RESPONSE_REPLACE_ALL;
+          else if (response == THUNAR_JOB_RESPONSE_RENAME)
+            response = THUNAR_JOB_RESPONSE_RENAME_ALL;
+        }
+    }
+
   g_signal_handlers_disconnect_by_data (src_file, src_image);
   g_signal_handlers_disconnect_by_data (dst_file, dst_image);
   gtk_widget_destroy (dialog);
@@ -1030,10 +1029,6 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   /* cleanup */
   g_object_unref (G_OBJECT (icon_factory));
 
-  /* translate GTK responses */
-  if (G_UNLIKELY (response < 0))
-    response = THUNAR_JOB_RESPONSE_CANCEL;
-
   return response;
 }
 
@@ -1045,14 +1040,14 @@ thunar_dialogs_show_insecure_program (gpointer     parent,
                                       ThunarFile  *file,
                                       const gchar *command)
 {
-  GdkScreen      *screen;
-  GtkWindow      *window;
-  gint            response;
-  GtkWidget      *dialog;
-  GString        *secondary;
-  GError         *err = NULL;
-  gchar          *file_name;
-  gchar          *executable;
+  GdkScreen *screen;
+  GtkWindow *window;
+  gint       response;
+  GtkWidget *dialog;
+  GString   *secondary;
+  GError    *err = NULL;
+  gchar     *file_name;
+  gchar     *executable;
 
   _thunar_return_val_if_fail (THUNAR_IS_FILE (file), FALSE);
   _thunar_return_val_if_fail (g_utf8_validate (command, -1, NULL), FALSE);
@@ -1065,33 +1060,32 @@ thunar_dialogs_show_insecure_program (gpointer     parent,
     file_name = g_strdup_printf ("\"%s\" (%s)", thunar_file_get_display_name (file), thunar_file_get_basename (file));
   else
     file_name = g_strdup_printf ("\"%s\"", thunar_file_get_display_name (file));
-    
+
   if (g_file_info_get_attribute_boolean (thunar_file_get_info (file), G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE) == FALSE)
     /* TRANSLATORS: This string will be inserted in the following sentence in the second appearance of %s:
      * "The desktop file %s is in an insecure location and not marked as secure%s." */
     executable = g_strdup (_("/executable"));
   else
     executable = g_strdup ("");
-    
+
   secondary = g_string_new (NULL);
   g_string_printf (secondary,
                    _("The desktop file %s is in an insecure location and not marked as secure%s. "
                      "If you do not trust this program, click Cancel.\n\n"),
                    file_name,
                    executable);
-  
+
   g_free (file_name);
   g_free (executable);
-  
+
   if (g_uri_is_valid (command, G_URI_FLAGS_NONE, NULL))
-    g_string_append_printf (secondary, G_KEY_FILE_DESKTOP_KEY_URL"=%s", command);
+    g_string_append_printf (secondary, G_KEY_FILE_DESKTOP_KEY_URL "=%s", command);
   else
-    g_string_append_printf (secondary, G_KEY_FILE_DESKTOP_KEY_EXEC"=%s", command);
+    g_string_append_printf (secondary, G_KEY_FILE_DESKTOP_KEY_EXEC "=%s", command);
 
   /* allocate and display the error message dialog */
   dialog = gtk_message_dialog_new (window,
-                                   GTK_DIALOG_MODAL |
-                                   GTK_DIALOG_DESTROY_WITH_PARENT,
+                                   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_WARNING,
                                    GTK_BUTTONS_NONE,
                                    "%s", primary);
@@ -1133,12 +1127,13 @@ thunar_dialogs_show_insecure_program (gpointer     parent,
 
 
 
-static void thunar_dialogs_select_filename (GtkWidget *entry)
+static void
+thunar_dialogs_select_filename (GtkWidget *entry)
 {
   const gchar    *filename;
   const gchar    *ext;
   glong           offset;
-  GtkEntryBuffer* buffer;
+  GtkEntryBuffer *buffer;
 
   buffer = gtk_entry_get_buffer (GTK_ENTRY (entry));
   filename = gtk_entry_buffer_get_text (buffer);
@@ -1212,7 +1207,7 @@ thunar_dialog_confirm_close_split_pane_tabs (GtkWindow *parent)
       g_object_set (G_OBJECT (preferences), "misc-confirm-close-multiple-tabs", FALSE, NULL);
       g_object_unref (G_OBJECT (preferences));
     }
-    
+
   gtk_widget_destroy (dialog);
   g_free (secondary_text);
   return response;
@@ -1321,14 +1316,14 @@ thunar_dialog_show_launcher_props (ThunarFile *launcher,
                                    gpointer    parent)
 {
   const gchar *display_name;
-  gchar       *cmd   = NULL,
-              *uri   = NULL;
-  GError      *error = NULL;
-  GdkScreen   *screen;
+  gchar       *cmd = NULL,
+        *uri = NULL;
+  GError    *error = NULL;
+  GdkScreen *screen;
 
   _thunar_return_if_fail (launcher == NULL || THUNAR_IS_FILE (launcher));
   _thunar_return_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WIDGET (parent));
-  
+
   screen = thunar_util_parse_parent (parent, NULL);
   uri = thunar_file_dup_uri (launcher);
   display_name = gdk_display_get_name (gdk_screen_get_display (screen));
