@@ -18,25 +18,60 @@
  * Boston, MA 02110-1301, USA.
  */
 
+#ifdef __linux__
+#define _GNU_SOURCE
+#endif
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+
 #include <gio/gio.h>
 
 #ifdef HAVE_GIO_UNIX
-#include <gio/gunixmounts.h>
-#include <gio/gdesktopappinfo.h>
-#endif
 
-#include <libxfce4util/libxfce4util.h>
+#ifdef __linux__
+
+#define HAVE_DIRECT_IO 1
+#include <fcntl.h>
+#include <gio/gunixinputstream.h>
+
+#if HAVE_STATX
+#include <sys/stat.h>
+#ifdef STATX_DIOALIGN
+#define HAVE_STATX_DIOALIGN 1
+#endif /* STATX_DIOALIGN */
+#endif /* HAVE_STATX */
+
+#endif /* __linux__ */
+
+#include <gio/gdesktopappinfo.h>
+#include <gio/gunixmounts.h>
+
+#endif /* HAVE_GIO_UNIX */
 
 #include "thunar/thunar-file.h"
 #include "thunar/thunar-gio-extensions.h"
 #include "thunar/thunar-preferences.h"
 #include "thunar/thunar-private.h"
 #include "thunar/thunar-util.h"
-#include "thunar/thunar-file.h"
+
+#include <libxfce4util/libxfce4util.h>
+
+#define CMP_BUF_MIN_ALIGN (16)
+#define CMP_BUF_SIZE (1024 * 512)
+
+#ifndef O_BINARY
+#define O_BINARY (0)
+#endif
 
 
 
@@ -46,6 +81,8 @@ static struct
   const gchar *icon_name;
   const gchar *type;
 }
+
+/* clang-format off */
 device_icon_name [] =
 {
   /* Implementation specific */
@@ -71,16 +108,20 @@ device_icon_name [] =
   { "phone*"                 , N_("Phone") },
   { NULL                     , NULL }
 };
+/* clang-format on */
 
 
 
-static const gchar     *guess_device_type_from_icon_name           (const gchar *icon_name);
-static       GFileInfo *thunar_g_file_get_content_type_querry_info (GFile       *gfile,
-                                                                    GError      *err);
-static       void       thunar_g_file_info_set_attribute           (GFileInfo   *info,
-                                                                    ThunarGType  type,
-                                                                    const gchar *setting_name,
-                                                                    const gchar *setting_value);
+static const gchar *
+guess_device_type_from_icon_name (const gchar *icon_name);
+static GFileInfo *
+thunar_g_file_get_content_type_querry_info (GFile  *gfile,
+                                            GError *err);
+static void
+thunar_g_file_info_set_attribute (GFileInfo   *info,
+                                  ThunarGType  type,
+                                  const gchar *setting_name,
+                                  const gchar *setting_value);
 
 GFile *
 thunar_g_file_new_for_home (void)
@@ -165,16 +206,16 @@ thunar_g_file_new_for_bookmarks (void)
 GFile *
 thunar_g_file_resolve_symlink (GFile *file)
 {
-  gchar       *basename;
-  GFile       *parent = NULL;
-  GFile       *target = NULL;
+  gchar *basename;
+  GFile *parent = NULL;
+  GFile *target = NULL;
 
   _thunar_return_val_if_fail (G_IS_FILE (file), NULL);
 
   parent = g_file_get_parent (file);
   if (parent == NULL)
     return NULL;
-  
+
   basename = g_file_get_basename (file);
   if (basename == NULL)
     {
@@ -246,7 +287,7 @@ thunar_g_file_is_home (GFile *file)
 gboolean
 thunar_g_file_is_trash (GFile *file)
 {
-  char *uri;
+  char    *uri;
   gboolean is_trash = FALSE;
 
   _thunar_return_val_if_fail (G_IS_FILE (file), FALSE);
@@ -263,7 +304,7 @@ thunar_g_file_is_trash (GFile *file)
 gboolean
 thunar_g_file_is_recent (GFile *file)
 {
-  char *uri;
+  char    *uri;
   gboolean is_recent = FALSE;
 
   _thunar_return_val_if_fail (G_IS_FILE (file), FALSE);
@@ -280,7 +321,7 @@ thunar_g_file_is_recent (GFile *file)
 gboolean
 thunar_g_file_is_computer (GFile *file)
 {
-  char *uri;
+  char    *uri;
   gboolean is_computer = FALSE;
 
   _thunar_return_val_if_fail (G_IS_FILE (file), FALSE);
@@ -297,7 +338,7 @@ thunar_g_file_is_computer (GFile *file)
 gboolean
 thunar_g_file_is_network (GFile *file)
 {
-  char *uri;
+  char    *uri;
   gboolean is_network = FALSE;
 
   _thunar_return_val_if_fail (G_IS_FILE (file), FALSE);
@@ -312,9 +353,9 @@ thunar_g_file_is_network (GFile *file)
 
 
 GKeyFile *
-thunar_g_file_query_key_file (GFile              *file,
-                              GCancellable       *cancellable,
-                              GError            **error)
+thunar_g_file_query_key_file (GFile        *file,
+                              GCancellable *cancellable,
+                              GError      **error)
 {
   GKeyFile *key_file;
   gchar    *contents = NULL;
@@ -333,10 +374,10 @@ thunar_g_file_query_key_file (GFile              *file,
 
   /* try to parse the key file from the contents of the file */
   if (G_LIKELY (length == 0
-      || g_key_file_load_from_data (key_file, contents, length,
-                                    G_KEY_FILE_KEEP_COMMENTS
-                                    | G_KEY_FILE_KEEP_TRANSLATIONS,
-                                    error)))
+                || g_key_file_load_from_data (key_file, contents, length,
+                                              G_KEY_FILE_KEEP_COMMENTS
+                                              | G_KEY_FILE_KEEP_TRANSLATIONS,
+                                              error)))
     {
       g_free (contents);
       return key_file;
@@ -357,9 +398,9 @@ thunar_g_file_write_key_file (GFile        *file,
                               GCancellable *cancellable,
                               GError      **error)
 {
-  gchar    *contents;
-  gsize     length;
-  gboolean  result = TRUE;
+  gchar   *contents;
+  gsize    length;
+  gboolean result = TRUE;
 
   _thunar_return_val_if_fail (G_IS_FILE (file), FALSE);
   _thunar_return_val_if_fail (key_file != NULL, FALSE);
@@ -402,9 +443,9 @@ thunar_g_file_get_location (GFile *file)
 
 
 static const gchar *
-guess_device_type_from_icon_name (const gchar * icon_name)
+guess_device_type_from_icon_name (const gchar *icon_name)
 {
-  for (int i = 0; device_icon_name[i].type != NULL ; i++)
+  for (int i = 0; device_icon_name[i].type != NULL; i++)
     {
       if (g_pattern_match_simple (device_icon_name[i].icon_name, icon_name))
         return g_dgettext (NULL, device_icon_name[i].type);
@@ -423,10 +464,10 @@ guess_device_type_from_icon_name (const gchar * icon_name)
 const gchar *
 thunar_g_file_guess_device_type (GFile *file)
 {
-  GFileInfo         *fileinfo    = NULL;
-  GIcon             *icon        = NULL;
-  const gchar       *icon_name   = NULL;
-  const gchar       *device_type = NULL;
+  GFileInfo   *fileinfo = NULL;
+  GIcon       *icon = NULL;
+  const gchar *icon_name = NULL;
+  const gchar *device_type = NULL;
 
   _thunar_return_val_if_fail (G_IS_FILE (file), NULL);
 
@@ -473,11 +514,11 @@ thunar_g_file_get_display_name (GFile *file)
         display_name = g_uri_escape_string (base_name, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, TRUE);
 
       g_free (base_name);
-   }
- else
-   {
-     display_name = g_strdup ("?");
-   }
+    }
+  else
+    {
+      display_name = g_strdup ("?");
+    }
 
   return display_name;
 }
@@ -525,8 +566,8 @@ thunar_g_file_get_display_name_remote (GFile *mount_point)
                 {
                   skip = strchr (p, skip_chars[n]);
                   if (skip != NULL
-                       && (path == NULL || skip < path)
-                       && (skip < firstdot))
+                      && (path == NULL || skip < path)
+                      && (skip < firstdot))
                     p = skip + 1;
                 }
             }
@@ -569,10 +610,10 @@ thunar_g_file_get_display_name_remote (GFile *mount_point)
 gboolean
 thunar_g_vfs_is_uri_scheme_supported (const gchar *scheme)
 {
-  const gchar * const *supported_schemes;
-  gboolean             supported = FALSE;
-  guint                n;
-  GVfs                *gvfs;
+  const gchar *const *supported_schemes;
+  gboolean            supported = FALSE;
+  guint               n;
+  GVfs               *gvfs;
 
   _thunar_return_val_if_fail (scheme != NULL && *scheme != '\0', FALSE);
 
@@ -644,11 +685,11 @@ thunar_g_file_get_free_space (GFile   *file,
 gchar *
 thunar_g_file_get_free_space_string (GFile *file, gboolean file_size_binary)
 {
-  gchar             *fs_size_free_str;
-  gchar             *fs_size_used_str;
-  guint64            fs_size_free;
-  guint64            fs_size_total;
-  gchar             *free_space_string = NULL;
+  gchar  *fs_size_free_str;
+  gchar  *fs_size_used_str;
+  guint64 fs_size_free;
+  guint64 fs_size_total;
+  gchar  *free_space_string = NULL;
 
   _thunar_return_val_if_fail (G_IS_FILE (file), NULL);
 
@@ -658,8 +699,8 @@ thunar_g_file_get_free_space_string (GFile *file, gboolean file_size_binary)
       fs_size_used_str = g_format_size_full (fs_size_total - fs_size_free, file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT);
 
       free_space_string = g_strdup_printf (_("%s used (%.0f%%)  |  %s free (%.0f%%)"),
-                                   fs_size_used_str, ((fs_size_total - fs_size_free) * 100.0 / fs_size_total),
-                                   fs_size_free_str, (fs_size_free * 100.0 / fs_size_total));
+                                           fs_size_used_str, ((fs_size_total - fs_size_free) * 100.0 / fs_size_total),
+                                           fs_size_free_str, (fs_size_free * 100.0 / fs_size_total));
 
       g_free (fs_size_free_str);
       g_free (fs_size_used_str);
@@ -677,7 +718,7 @@ thunar_g_file_list_get_type (void)
 
   if (G_UNLIKELY (type == G_TYPE_INVALID))
     {
-      type = g_boxed_type_register_static (I_("ThunarGFileList"),
+      type = g_boxed_type_register_static (I_ ("ThunarGFileList"),
                                            (GBoxedCopyFunc) thunar_g_list_copy_deep,
                                            (GBoxedFreeFunc) thunar_g_list_free_full);
     }
@@ -687,7 +728,7 @@ thunar_g_file_list_get_type (void)
 
 
 
-static GHashTable*
+static GHashTable *
 thunar_g_hastable_copy_deep (GHashTable *table)
 {
   GHashTable    *copy = g_hash_table_new_full (g_direct_hash, NULL, g_object_unref, NULL);
@@ -709,7 +750,7 @@ thunar_g_file_hastable_get_type (void)
 
   if (G_UNLIKELY (type == G_TYPE_INVALID))
     {
-      type = g_boxed_type_register_static (I_("ThunarGHashTableList"),
+      type = g_boxed_type_register_static (I_ ("ThunarGHashTableList"),
                                            (GBoxedCopyFunc) thunar_g_hastable_copy_deep,
                                            (GBoxedFreeFunc) g_hash_table_destroy);
     }
@@ -807,7 +848,7 @@ thunar_g_file_copy (GFile                *source,
     }
 
   /* generate partial file name */
-  base_name    = g_file_get_basename (destination);
+  base_name = g_file_get_basename (destination);
   if (base_name == NULL)
     {
       base_name = g_strdup ("UNNAMED");
@@ -815,10 +856,10 @@ thunar_g_file_copy (GFile                *source,
 
   /* limit filename length */
   partial_name = g_strdup_printf ("%.100s.partial~", base_name);
-  parent       = g_file_get_parent (destination);
+  parent = g_file_get_parent (destination);
 
   /* parent can't be NULL since destination must be a file */
-  partial      = g_file_get_child (parent, partial_name);
+  partial = g_file_get_child (parent, partial_name);
   g_clear_object (&parent);
   g_free (partial_name);
 
@@ -832,7 +873,10 @@ thunar_g_file_copy (GFile                *source,
   if (success)
     {
       /* rename .partial if done without problem */
-      success = (g_file_set_display_name (partial, base_name, NULL, error) != NULL);
+      GFile *renamed_file = g_file_set_display_name (partial, base_name, NULL, error);
+      success = (renamed_file != NULL);
+      if (success)
+        g_object_unref (renamed_file);
     }
 
   if (!success)
@@ -851,36 +895,266 @@ thunar_g_file_copy (GFile                *source,
 
 
 
+#ifdef HAVE_DIRECT_IO
+static GInputStream *
+open_file_and_buffer_for_direct_io (GFile        *file,
+                                    GCancellable *cancellable,
+                                    void        **buf_out,
+                                    unsigned int *buf_align_out,
+                                    size_t       *buf_size_out)
+{
+  int    fd = -1;
+  size_t buf_align = 0;
+  size_t offset_align = 0;
+  size_t buf_size;
+  void  *buf = NULL;
+#ifdef HAVE_STATX_DIOALIGN
+  struct statx statx_buf;
+
+  // First we use statx () to ask the filesystem the required buffer alignment
+  // for direct I/O (stx_dio_mem_align), as well as what the file offset
+  // alignment (stx_dio_offset_align), which tells us what the buffer size is
+  // required to be a multiple of.
+  DBG ("Trying statx ()");
+  if (statx (-1, g_file_peek_path (file), AT_STATX_SYNC_AS_STAT, STATX_DIOALIGN, &statx_buf) == 0
+      && statx_buf.stx_dio_mem_align > 0
+      && statx_buf.stx_dio_offset_align > 0)
+    {
+      DBG ("statx () succeeded, direct I/O align=%u, offset_align=%u",
+           statx_buf.stx_dio_mem_align,
+           statx_buf.stx_dio_offset_align);
+      // Ensure alignment is at least 16, which is required for most SIMD
+      // operations.  If somehow the required filesystem alignment/offset
+      // alignment is lower than 16, it's always safe to go higher.
+      buf_align = MAX (CMP_BUF_MIN_ALIGN, statx_buf.stx_dio_mem_align);
+      offset_align = MAX (CMP_BUF_MIN_ALIGN, statx_buf.stx_dio_offset_align);
+    }
+#endif
+
+  if (buf_align == 0 || offset_align == 0)
+    {
+      DBG ("statx () not supported or failed; using alignment/offset alignment fallback of 512");
+      // Generally this fallback value should work on linux 2.6 & above.
+      buf_align = 512;
+      offset_align = 512;
+    }
+
+  // Since we're using blocking stdio operations and not cancellable GIO
+  // operations, we should check the cancellable periodically.
+  if (cancellable != NULL && g_cancellable_is_cancelled (cancellable))
+    {
+      goto out_err;
+    }
+
+  // Ensure the buffer size is a multiple of offset_align.
+  buf_size = (CMP_BUF_SIZE / offset_align) * offset_align;
+  DBG ("buf_size will be %" G_GSIZE_FORMAT " based on offset_align", buf_size);
+  if (buf_size == 0)
+    {
+      // This shouldn't happen, but if somehow we get a truly bizarre
+      // offst_align value that is larger than CMP_BUF_SIZE, we'll just bail.
+      goto out_err;
+    }
+
+  // Open using the same flags g_file_read () uses, plus O_DIRECT.
+  fd = open (g_file_peek_path (file), O_RDONLY | O_BINARY | O_CLOEXEC | O_DIRECT, 0);
+  if (fd == -1 || (cancellable != NULL && g_cancellable_is_cancelled (cancellable)))
+    {
+      goto out_err;
+    }
+  DBG ("Successfully opened file with O_DIRECT");
+
+  // Create a buffer to read data into, with the required alignment and size.
+  if (posix_memalign (&buf, buf_align, buf_size) != 0)
+    {
+      goto out_err;
+    }
+  DBG ("Successfully allocated memory for direct I/O buffers");
+
+  // Even if we've successfully opened the file for direct I/O and have
+  // allocated our buffer, it's still possible that there's something wrong
+  // with our buffer size or alignment.  So do a test read to see if it
+  // succeeds, and if so, seek back to the beginning of the file.
+  if (read (fd, buf, buf_size) == -1
+      || lseek (fd, 0, SEEK_SET) == (off_t) -1
+      || (cancellable != NULL && g_cancellable_is_cancelled (cancellable)))
+    {
+      goto out_err;
+    }
+  DBG ("Direct I/O is available and seems to be working");
+
+  *buf_out = buf;
+  *buf_align_out = buf_align;
+  *buf_size_out = buf_size;
+
+  return g_unix_input_stream_new (fd, TRUE);
+
+out_err:
+
+  if (fd >= 0)
+    {
+      close (fd);
+    }
+  free (buf);
+
+  return NULL;
+}
+#endif
+
+
+
+static GInputStream *
+open_file_and_buffer_fallback (GFile        *file,
+                               GCancellable *cancellable,
+                               void        **buf_out,
+                               unsigned int *buf_align_out,
+                               size_t       *buf_size_out,
+                               GError      **error)
+{
+  GFileInputStream *finp;
+  void             *buf = NULL;
+
+  finp = g_file_read (file, cancellable, error);
+
+  if (finp != NULL)
+    {
+      /* Use 16-byte aligned allocations (rather than allocating on the stack), as
+         that's usually a requirement for any SIMD optimizations the compiler might
+         be able to do. */
+      if (posix_memalign (&buf, CMP_BUF_MIN_ALIGN, CMP_BUF_SIZE) != 0)
+        {
+          g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errno), "Failed to allocate aligned memory: %s", strerror (errno));
+        }
+    }
+
+  if (finp != NULL && buf != NULL)
+    {
+      *buf_out = buf;
+      *buf_align_out = CMP_BUF_MIN_ALIGN;
+      *buf_size_out = CMP_BUF_SIZE;
+
+      return G_INPUT_STREAM (finp);
+    }
+  else
+    {
+      if (finp != NULL)
+        {
+          g_object_unref (finp);
+        }
+      free (buf);
+
+      return FALSE;
+    }
+}
+
+
+
 /**
- * thunar_g_file_compare_checksum:
+ * thunar_g_file_compare_contents:
  * @file_a      : a #GFile
  * @file_b      : a #GFile
  * @cancellable : (nullalble): optional #GCancellable object
  * @error       : (nullalble): optional #GError
  *
- * Compare @file_a and @file_b with checksum.
+ * Compares the contents of @file_a and @file_b.
  *
- * Return value: %TRUE if a checksum matches, %FALSE if not.
+ * Return value: %TRUE if the files match, %FALSE if not.
  **/
 gboolean
-thunar_g_file_compare_checksum (GFile        *file_a,
+thunar_g_file_compare_contents (GFile        *file_a,
                                 GFile        *file_b,
                                 GCancellable *cancellable,
                                 GError      **error)
 {
-  gchar   *str_a;
-  gchar   *str_b;
-  gboolean is_equal;
+  GFileInputStream *inp_a = NULL;
+  GInputStream     *inp_b = NULL;
+  void             *buf_a = NULL;
+  void             *buf_b = NULL;
+  unsigned int      buf_align = 0;
+  size_t            buf_size = 0;
+  gboolean          is_equal = FALSE;
 
+  g_return_val_if_fail (G_IS_FILE (file_a), FALSE);
+  g_return_val_if_fail (G_IS_FILE (file_b), FALSE);
+  g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  str_a = xfce_g_file_create_checksum (file_a, cancellable, error);
-  str_b = xfce_g_file_create_checksum (file_b, cancellable, error);
+  // We assume that file_b is the destination file, and that's the one that
+  // really needs to be opened using O_DIRECT, as it's just been written, and
+  // some (or even all) of it will still be in the kernel's buffer cache.
+  // file_a can be opened normally, and will hopefully/likely be read from
+  // buffer cache, which is both fine and faster.
 
-  is_equal = g_strcmp0 (str_a, str_b) == 0;
+  inp_a = g_file_read (file_a, cancellable, error);
 
-  g_free (str_a);
-  g_free (str_b);
+  if (inp_a != NULL)
+    {
+#ifdef HAVE_DIRECT_IO
+      if (g_file_has_uri_scheme (file_b, "file"))
+        {
+          DBG ("Attempting direct I/O for destination file");
+          inp_b = open_file_and_buffer_for_direct_io (file_b, cancellable, &buf_b, &buf_align, &buf_size);
+        }
+#endif
+
+      if (inp_b == NULL)
+        {
+          inp_b = open_file_and_buffer_fallback (file_b, cancellable, &buf_b, &buf_align, &buf_size, error);
+        }
+    }
+
+  if (inp_b != NULL)
+    {
+      // Now that we know the alignment and size needed for file_b, we can also
+      // allocate the buffer for file_a.
+      if (posix_memalign (&buf_a, buf_align, buf_size) != 0)
+        {
+          g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errno), "Failed to allocate aligned memory: %s", strerror (errno));
+        }
+    }
+
+  if (inp_a != NULL && inp_b != NULL && buf_a != NULL)
+    {
+      for (;;)
+        {
+          gssize bytes_read_a = g_input_stream_read (G_INPUT_STREAM (inp_a), buf_a, buf_size, cancellable, error);
+          if (bytes_read_a < 0)
+            {
+              break;
+            }
+          gssize bytes_read_b = g_input_stream_read (G_INPUT_STREAM (inp_b), buf_b, buf_size, cancellable, error);
+          if (bytes_read_b < 0)
+            {
+              break;
+            }
+
+          if (bytes_read_a == 0 && bytes_read_b == 0)
+            {
+              is_equal = TRUE;
+              break;
+            }
+          else if (bytes_read_a != bytes_read_b)
+            {
+              break;
+            }
+          else if (memcmp (buf_a, buf_b, bytes_read_a) != 0)
+            {
+              break;
+            }
+        }
+    }
+
+  if (inp_a != NULL)
+    {
+      g_object_unref (inp_a);
+    }
+  if (inp_b != NULL)
+    {
+      g_object_unref (inp_b);
+    }
+
+  free (buf_a);
+  free (buf_b);
 
   return is_equal;
 }
@@ -950,7 +1224,7 @@ thunar_g_file_list_to_stringv (GList *list)
       else
         {
           uris[n++] = g_filename_to_uri (path, NULL, NULL);
-          g_free(path);
+          g_free (path);
         }
     }
 
@@ -969,14 +1243,14 @@ thunar_g_file_list_to_stringv (GList *list)
  *
  * Return value: A list of #GFile<!---->s of all parent folders. Free the returned list with calling g_object_unref() on each element
  **/
-GList*
+GList *
 thunar_g_file_list_get_parents (GList *file_list)
 {
-  GList    *lp_file_list;
-  GList    *lp_parent_folder_list;
-  GFile    *parent_folder;
-  GList    *parent_folder_list = NULL;
-  gboolean  folder_already_added;
+  GList   *lp_file_list;
+  GList   *lp_parent_folder_list;
+  GFile   *parent_folder;
+  GList   *parent_folder_list = NULL;
+  gboolean folder_already_added;
 
   for (lp_file_list = file_list; lp_file_list != NULL; lp_file_list = lp_file_list->next)
     {
@@ -1021,7 +1295,7 @@ thunar_g_file_is_descendant (GFile *descendant,
                              GFile *ancestor)
 {
   _thunar_return_val_if_fail (descendant != NULL && G_IS_FILE (descendant), FALSE);
-  _thunar_return_val_if_fail (ancestor   != NULL && G_IS_FILE (ancestor),   FALSE);
+  _thunar_return_val_if_fail (ancestor != NULL && G_IS_FILE (ancestor), FALSE);
 
   for (GFile *parent = g_object_ref (descendant), *temp;
        parent != NULL;
@@ -1046,15 +1320,15 @@ thunar_g_app_info_launch (GAppInfo          *info,
                           GAppLaunchContext *context,
                           GError           **error)
 {
-  ThunarFile   *file;
-  GAppInfo     *default_app_info;
-  GList        *recommended_app_infos;
-  GList        *lp;
-  const gchar  *content_type;
-  gboolean      result = FALSE;
-  gchar        *new_path = NULL;
-  gchar        *old_path = NULL;
-  gboolean      skip_app_info_update;
+  ThunarFile  *file;
+  GAppInfo    *default_app_info;
+  GList       *recommended_app_infos;
+  GList       *lp;
+  const gchar *content_type;
+  gboolean     result = FALSE;
+  gchar       *new_path = NULL;
+  gchar       *old_path = NULL;
+  gboolean     skip_app_info_update;
 
   _thunar_return_val_if_fail (G_IS_APP_INFO (info), FALSE);
   _thunar_return_val_if_fail (working_directory == NULL || G_IS_FILE (working_directory), FALSE);
@@ -1116,7 +1390,7 @@ thunar_g_app_info_launch (GAppInfo          *info,
                   if (g_app_info_equal (info, recommended_app_infos->data))
                     update_app_info = FALSE;
 
-                  g_list_free (recommended_app_infos);
+                  g_list_free_full (recommended_app_infos, g_object_unref);
                 }
             }
 
@@ -1220,19 +1494,19 @@ thunar_g_vfs_metadata_is_supported (void)
 gboolean
 thunar_g_file_is_on_local_device (GFile *file)
 {
-  gboolean  is_local = FALSE;
-  GFile    *target_file;
-  GFile    *target_parent;
-  GMount   *file_mount;
+  gboolean is_local = FALSE;
+  GFile   *target_file;
+  GFile   *target_parent;
+  GMount  *file_mount;
 
   _thunar_return_val_if_fail (file != NULL, TRUE);
   _thunar_return_val_if_fail (G_IS_FILE (file), TRUE);
 
   if (g_file_has_uri_scheme (file, "file") == FALSE)
     return FALSE;
-  for (target_file  = g_object_ref (file);
+  for (target_file = g_object_ref (file);
        target_file != NULL;
-       target_file  = target_parent)
+       target_file = target_parent)
     {
       if (g_file_query_exists (target_file, NULL))
         break;
@@ -1256,7 +1530,7 @@ thunar_g_file_is_on_local_device (GFile *file)
       /* mountpoints which cannot be unmounted are local devices.
        * attached devices like USB key/disk, fuse mounts, Samba shares,
        * PTP devices can always be unmounted and are considered remote/slow. */
-      is_local = ! g_mount_can_unmount (file_mount);
+      is_local = !g_mount_can_unmount (file_mount);
       g_object_unref (file_mount);
     }
 
@@ -1277,9 +1551,9 @@ gboolean
 thunar_g_file_set_executable_flags (GFile   *file,
                                     GError **error)
 {
-  ThunarFileMode  old_mode;
-  ThunarFileMode  new_mode;
-  GFileInfo      *info;
+  ThunarFileMode old_mode;
+  ThunarFileMode new_mode;
+  GFileInfo     *info;
 
   _thunar_return_val_if_fail (G_IS_FILE (file), FALSE);
   _thunar_return_val_if_fail (error == NULL || *error == NULL, FALSE);
@@ -1332,10 +1606,10 @@ thunar_g_file_set_executable_flags (GFile   *file,
 gboolean
 thunar_g_file_is_in_xdg_data_dir (GFile *file)
 {
-    const gchar * const *data_dirs;
-    guint                i;
-    gchar               *path;
-    gboolean             found = FALSE;
+  const gchar *const *data_dirs;
+  guint               i;
+  gchar              *path;
+  gboolean            found = FALSE;
 
   _thunar_return_val_if_fail (G_IS_FILE (file), FALSE);
 
@@ -1348,15 +1622,15 @@ thunar_g_file_is_in_xdg_data_dir (GFile *file)
           for (i = 0; data_dirs[i] != NULL; i++)
             {
               if (g_str_has_prefix (path, data_dirs[i]))
-              {
-                found = TRUE;
-                break;
-              }
+                {
+                  found = TRUE;
+                  break;
+                }
             }
           g_free (path);
         }
     }
-    return found;
+  return found;
 }
 
 
@@ -1388,7 +1662,7 @@ thunar_g_file_is_desktop_file (GFile *file)
         is_desktop_file = TRUE;
       g_object_unref (info);
     }
-  
+
   g_free (basename);
   return is_desktop_file;
 }
@@ -1409,49 +1683,46 @@ char *
 thunar_g_file_get_link_path_for_symlink (GFile *file_to_link,
                                          GFile *symlink)
 {
-    GFile *root;
-    GFile *parent;
-    char  *relative_path;
-    char  *link_path;
+  GFile *root;
+  GFile *parent;
+  char  *relative_path;
+  char  *link_path;
 
   _thunar_return_val_if_fail (G_IS_FILE (file_to_link), NULL);
   _thunar_return_val_if_fail (G_IS_FILE (symlink), NULL);
 
-    /* */
-    if (g_file_is_native (file_to_link) || g_file_is_native (symlink))
+  /* */
+  if (g_file_is_native (file_to_link) || g_file_is_native (symlink))
     {
-        return g_file_get_path (file_to_link);
+      return g_file_get_path (file_to_link);
     }
 
-    /* Search for the filesystem root */
-    root = g_object_ref (file_to_link);
-    while ((parent = g_file_get_parent (root)) != NULL)
+  /* Search for the filesystem root */
+  root = g_object_ref (file_to_link);
+  while ((parent = g_file_get_parent (root)) != NULL)
     {
-        g_object_unref (root);
-        root = parent;
+      g_object_unref (root);
+      root = parent;
     }
 
-    /* Build a absolute path, using the relative path up to the filesystem root */
-    relative_path = g_file_get_relative_path (root, file_to_link);
-    g_object_unref (root);
-    link_path = g_strconcat ("/", relative_path, NULL);
-    g_free (relative_path);
-    return link_path;
+  /* Build a absolute path, using the relative path up to the filesystem root */
+  relative_path = g_file_get_relative_path (root, file_to_link);
+  g_object_unref (root);
+  link_path = g_strconcat ("/", relative_path, NULL);
+  g_free (relative_path);
+  return link_path;
 }
 
 
 
-static GFileInfo*
-thunar_g_file_get_content_type_querry_info (GFile *gfile,
+static GFileInfo *
+thunar_g_file_get_content_type_querry_info (GFile  *gfile,
                                             GError *err)
 {
-  GFileInfo   *info = NULL;
+  GFileInfo *info = NULL;
 
   info = g_file_query_info (gfile,
-                            G_FILE_ATTRIBUTE_STANDARD_TYPE ","
-                            G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK ","
-                            G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
-                            G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE,
+                            G_FILE_ATTRIBUTE_STANDARD_TYPE "," G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK "," G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE "," G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE,
                             G_FILE_QUERY_INFO_NONE,
                             NULL, &err);
   return info;
@@ -1465,16 +1736,16 @@ thunar_g_file_get_content_type_querry_info (GFile *gfile,
  *
  * Gets the content type of the passed #GFile. Will always return a valid string.
  *
- * Return value: (transfer full): The content type as #gchar 
+ * Return value: (transfer full): The content type as #gchar
  * The returned string should be freed with g_free() when no longer needed.
  **/
-char*
+char *
 thunar_g_file_get_content_type (GFile *gfile)
 {
-  gboolean     is_symlink = FALSE;
-  GFileInfo   *info = NULL;
-  GError      *err = NULL;
-  gchar       *content_type = NULL;
+  gboolean   is_symlink = FALSE;
+  GFileInfo *info = NULL;
+  GError    *err = NULL;
+  gchar     *content_type = NULL;
 
   info = thunar_g_file_get_content_type_querry_info (gfile, err);
 
@@ -1512,17 +1783,17 @@ thunar_g_file_get_content_type (GFile *gfile)
   else
     {
       /* If gfile retrieved above is NULL, then g_file_query_info won't be called, thus keeping info NULL.
-        * In this case, err will also be NULL. So it will fallback to "unknown" mime-type */
+       * In this case, err will also be NULL. So it will fallback to "unknown" mime-type */
       if (G_LIKELY (err != NULL))
         {
           /* The mime-type 'inode/symlink' is  only used for broken links.
-            * When the link is functional, the mime-type of the link target will be used */
+           * When the link is functional, the mime-type of the link target will be used */
           if (G_LIKELY (is_symlink && err->code == G_IO_ERROR_NOT_FOUND))
-              content_type = g_strdup ("inode/symlink");
+            content_type = g_strdup ("inode/symlink");
           else
-              g_warning ("Content type loading failed for %s: %s",
-                          g_file_get_uri (gfile),
-                          err->message);
+            g_warning ("Content type loading failed for %s: %s",
+                       g_file_get_uri (gfile),
+                       err->message);
           g_error_free (err);
         }
     }
@@ -1575,11 +1846,11 @@ thunar_g_file_info_set_attribute (GFileInfo   *info,
 {
   switch (type)
     {
-      case THUNAR_GTYPE_STRING:
-        g_file_info_set_attribute_string (info, setting_name, setting_value);
-        break;
+    case THUNAR_GTYPE_STRING:
+      g_file_info_set_attribute_string (info, setting_name, setting_value);
+      break;
 
-      case THUNAR_GTYPE_STRINGV:
+    case THUNAR_GTYPE_STRINGV:
       {
         gchar **setting_values;
         setting_values = g_strsplit (setting_value, THUNAR_METADATA_STRING_DELIMETER, 100);
@@ -1588,25 +1859,25 @@ thunar_g_file_info_set_attribute (GFileInfo   *info,
         break;
       }
 
-      default:
-        g_warning ("ThunarGType not supported, skipping");
-        break;
+    default:
+      g_warning ("ThunarGType not supported, skipping");
+      break;
     }
 }
 
 
 
-static gchar*
+static gchar *
 thunar_g_file_info_get_attribute (GFileInfo   *info,
                                   ThunarGType  type,
                                   const gchar *setting_name)
 {
   switch (type)
     {
-      case THUNAR_GTYPE_STRING:
-        return g_strdup (g_file_info_get_attribute_string (info, setting_name));
+    case THUNAR_GTYPE_STRING:
+      return g_strdup (g_file_info_get_attribute_string (info, setting_name));
 
-      case THUNAR_GTYPE_STRINGV:
+    case THUNAR_GTYPE_STRINGV:
       {
         gchar **stringv = g_file_info_get_attribute_stringv (info, setting_name);
         GList  *string_list = NULL;
@@ -1623,9 +1894,9 @@ thunar_g_file_info_get_attribute (GFileInfo   *info,
         return joined_string;
       }
 
-      default:
-        g_warning ("ThunarGType not supported, skipping");
-        return NULL;
+    default:
+      g_warning ("ThunarGType not supported, skipping");
+      return NULL;
     }
 }
 
